//Calendar implementation for the HTWK Leipzig timetable. Evaluation and display of the individual dates in iCal format.
//Copyright (C) 2024 HTWKalender support@htwkalender.de

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU Affero General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Affero General Public License for more details.

//You should have received a copy of the GNU Affero General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

package v2

import (
	"htwkalender/data-manager/model"
	"reflect"
	"testing"
)

func TestSwitchNameAndNotesForExam(t *testing.T) {
	type args struct {
		events []model.Event
	}
	tests := []struct {
		name string
		args args
		want []model.Event
	}{
		{
			name: "switch name and notes for exam",
			args: args{
				events: []model.Event{
					{
						EventType: "Pruefung",
						Name:      "Prüfungen FING/EIT WiSe (pruefungsamt.fing-eit@htwk-leipzig.de)",
						Notes:     "Computer Vision II - Räume/Zeit unter Vorbehalt- (Raum W111.1)",
					},
				},
			},
			want: []model.Event{
				{
					EventType: "Pruefung",
					Name:      "Computer Vision II - Räume/Zeit unter Vorbehalt- (Raum W111.1)",
					Notes:     "Prüfungen FING/EIT WiSe (pruefungsamt.fing-eit@htwk-leipzig.de)",
				},
			},
		},
		{
			name: "dont switch name and notes for exam",
			args: args{
				events: []model.Event{
					{
						EventType: "Pruefung",
						Name:      "i054 Umweltschutz und Recycling DPB & VNB 7.FS (wpf)",
						Notes:     "Prüfung",
					},
				},
			},
			want: []model.Event{
				{
					EventType: "Pruefung",
					Notes:     "Prüfung",
					Name:      "i054 Umweltschutz und Recycling DPB & VNB 7.FS (wpf)",
				},
			},
		},
		{
			name: "dont switch name and notes for exam",
			args: args{
				events: []model.Event{
					{
						EventType: "Pruefung",
						Name:      "Prüfungen FING/ME WiSe (pruefungsamt.fing-me@htwk-leipzig.de)",
						Notes:     "",
					},
				},
			},
			want: []model.Event{
				{
					EventType: "Pruefung",
					Notes:     "",
					Name:      "Prüfungen FING/ME WiSe (pruefungsamt.fing-me@htwk-leipzig.de)",
				},
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := switchNameAndNotesForExam(tt.args.events); !reflect.DeepEqual(got, tt.want) {
				t.Errorf("switchNameAndNotesForExam() = %v, want %v", got, tt.want)
			}
		})
	}
}

func Test_generateUUIDs(t *testing.T) {
	type args struct {
		events []model.Event
	}
	tests := []struct {
		name string
		args args
		want []model.Event
	}{
		{
			name: "generate UUIDs for events",
			args: args{
				events: []model.Event{
					{
						EventType: "Vorlesung",
						Name:      "Computer Vision II",
						Course:    "Computer Vision",
						UUID:      "",
					},
					{
						EventType: "Pruefung",
						Name:      "C169  Digitale Bildverarbeitung MIM & INM 3. FS (wpf)",
						Course:    "23INM",
						UUID:      "",
					},
					{
						EventType: "Vorlesung",
						Name:      "C398 Visualisierung in NW und Technik  IN-M & MI-M  2. FS (wpf)",
						Course:    "24INM",
						UUID:      "",
					},
					{
						EventType: "Vorlesung",
						Name:      "M947 Fluidenergiemaschinen EGB (pf) & MBB & SGB (wpf) 4.FS",
						Course:    "23EGB-EGTa",
						UUID:      "",
					},
				},
			},
			want: []model.Event{
				{
					EventType: "Vorlesung",
					Name:      "Computer Vision II",
					Course:    "Computer Vision",
					UUID:      "8ddd913c-27f0-58b2-be17-e50f2851d482",
				},
				{
					EventType: "Pruefung",
					Name:      "C169  Digitale Bildverarbeitung MIM & INM 3. FS (wpf)",
					Course:    "23INM",
					UUID:      "2a35348d-63ce-511c-8580-893321d104b9",
				},
				{
					EventType: "Vorlesung",
					Name:      "C398 Visualisierung in NW und Technik  IN-M & MI-M  2. FS (wpf)",
					Course:    "24INM",
					UUID:      "6051ebd9-dd2b-5646-82c4-091667f414ee",
				},
				{
					EventType: "Vorlesung",
					Name:      "M947 Fluidenergiemaschinen EGB (pf) & MBB & SGB (wpf) 4.FS",
					Course:    "23EGB-EGTa",
					UUID:      "736764e1-fa78-5195-8280-76c996dc8b47",
				},
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := generateUUIDs(tt.args.events); !reflect.DeepEqual(got, tt.want) {
				t.Errorf("generateUUIDs() = %v, want %v", got, tt.want)
			}
		})
	}
}
